@@ -46,7 +46,7 @@ extern int acpid_debug;
 extern int logevents;
 extern const char *progname;
 
-extern int locked();
+extern int locked(void);
 
 extern void clean_exit_with_status(int status);
 
