@@ -112,6 +112,8 @@ static struct evtab_entry evtab[] = {
  		"button/volumeup VOLUP 00000080 00000000"},
 	{{{0,0}, EV_KEY, KEY_MUTE, 1},
  		"button/mute MUTE 00000080 00000000"},
+	{{{0,0}, EV_KEY, KEY_MICMUTE, 1},
+		"button/micmute MICMUTE 00000080 00000000"},
  	/* cd play/pause buttons */
  	{{{0,0}, EV_KEY, KEY_NEXTSONG, 1},
  		"cd/next CDNEXT 00000080 00000000"},
@@ -409,7 +411,14 @@ int open_inputfile(const char *filename)
 			strcpy(c.pathname, filename);
 		/* assume not a keyboard until we see a scancode */
 		c.kybd = 0;
-		add_connection(&c);
+
+		if (add_connection(&c) < 0) {
+			close(fd);
+			acpid_log(LOG_ERR,
+				"can't add connection for input layer %s (%s)",
+				filename, evname);
+			return -1;
+		}
 
 		return 0;  /* success */
 	}
